@@ -7,9 +7,9 @@ class MetadataProvider:
     _metadata_class = Metadata
 
     @staticmethod
-    def get_metadata_provider(url):
+    def get_metadata_provider(url, account=None):
         """ Get an Metadata instance."""
-        return MetadataProvider._metadata_class(url)
+        return MetadataProvider._metadata_class(url, account)
 
     @staticmethod
     def set_metadata_class(metadata_class):
