@@ -32,6 +32,8 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     amounts = service_agreement.get_amounts_int()
     receivers = service_agreement.get_receivers()
     number_nfts = service_agreement.get_number_nfts()
+    nft_contract_address = service_agreement.get_nft_contract_address()
+    nft_transfer = service_agreement.get_nft_transfer_or_mint()
     token_address = keeper.token.address
 
     receiver_0_starting_balance = keeper.token.get_token_balance(
@@ -41,9 +43,9 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
         keeper.nft_sales_template.address), 'Template is not approved.'
     assert keeper.did_registry.get_block_number_updated(asset_id) > 0, 'asset id not registered'
     success = keeper.nft_sales_template.create_agreement(
-        agreement_id,
+        agreement_id[0],
         asset_id,
-        [lock_cond_id, access_cond_id, escrow_cond_id],
+        [lock_cond_id[0], access_cond_id[0], escrow_cond_id[0]],
         service_agreement.conditions_timelocks,
         service_agreement.conditions_timeouts,
         consumer_acc.address,
@@ -52,7 +54,7 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     print('create agreement: ', success)
     assert success, f'createAgreement failed {success}'
     event = keeper.nft_sales_template.subscribe_agreement_created(
-        agreement_id,
+        agreement_id[1],
         10,
         log_event(keeper.nft_sales_template.AGREEMENT_CREATED_EVENT),
         (),
@@ -61,10 +63,8 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     assert event, 'no event for AgreementCreated '
 
     # Verify condition types (condition contracts)
-    agreement = keeper.agreement_manager.get_agreement(agreement_id)
-    assert agreement.did == asset_id, ''
     cond_types = keeper.nft_sales_template.get_condition_types()
-    for i, cond_id in enumerate(agreement.condition_ids):
+    for i, cond_id in enumerate([lock_cond_id[1], access_cond_id[1], escrow_cond_id[1]]):
         cond = keeper.condition_manager.get_condition(cond_id)
         assert cond.type_ref == cond_types[i]
         assert int(cond.state) == 1
@@ -82,7 +82,7 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     keeper.token.token_approve(keeper.escrow_payment_condition.address, price, consumer_acc)
 
     tx_hash = keeper.lock_payment_condition.fulfill(
-        agreement_id,
+        agreement_id[1],
         asset_id,
         keeper.escrow_payment_condition.address,
         token_address,
@@ -91,14 +91,14 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
         consumer_acc)
     keeper.lock_payment_condition.get_tx_receipt(tx_hash)
     event = keeper.lock_payment_condition.subscribe_condition_fulfilled(
-        agreement_id,
+        agreement_id[1],
         10,
         log_event(keeper.lock_payment_condition.FULFILLED_EVENT),
         (),
         wait=True
     )
     assert event, 'no event for LockPaymentCondition.Fulfilled'
-    assert keeper.condition_manager.get_condition_state(lock_cond_id) == 2, ''
+    assert keeper.condition_manager.get_condition_state(lock_cond_id[1]) == 2, ''
     assert keeper.token.get_token_balance(
         keeper.escrow_payment_condition.address
     ) == (price + starting_balance), ''
@@ -110,34 +110,36 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
 
     # Fulfill transfer did condition
     tx_hash = keeper.transfer_nft_condition.fulfill(
-        agreement_id,
+        agreement_id[1],
         asset_id,
         consumer_acc.address,
         number_nfts,
-        lock_cond_id,
+        lock_cond_id[1],
+        nft_contract_address,
+        nft_transfer,
         publisher_acc
     )
     keeper.transfer_nft_condition.get_tx_receipt(tx_hash)
     event = keeper.transfer_nft_condition.subscribe_condition_fulfilled(
-        agreement_id,
+        agreement_id[1],
         20,
         log_event(keeper.transfer_nft_condition.FULFILLED_EVENT),
         (),
         wait=True
     )
     assert event, 'no event for TransferNFTCondition.Fulfilled'
-    assert keeper.condition_manager.get_condition_state(access_cond_id) == 2, ''
+    assert keeper.condition_manager.get_condition_state(access_cond_id[1]) == 2, ''
 
     # Fulfill escrow_payment_condition
     tx_hash = keeper.escrow_payment_condition.fulfill(
-        agreement_id, asset_id, amounts, receivers,
-        keeper.escrow_payment_condition.address, token_address, lock_cond_id,
-        access_cond_id, publisher_acc
+        agreement_id[1], asset_id, amounts, receivers, consumer_acc.address,
+        keeper.escrow_payment_condition.address, token_address, lock_cond_id[1],
+        access_cond_id[1], publisher_acc
     )
 
     keeper.escrow_payment_condition.get_tx_receipt(tx_hash)
     event = keeper.escrow_payment_condition.subscribe_condition_fulfilled(
-        agreement_id,
+        agreement_id[1],
         10,
         log_event(keeper.escrow_payment_condition.FULFILLED_EVENT),
         (),
@@ -145,7 +147,7 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     )
 
     assert event, 'no event for EscrowPayment.Fulfilled'
-    assert keeper.condition_manager.get_condition_state(escrow_cond_id) == 2, ''
+    assert keeper.condition_manager.get_condition_state(escrow_cond_id[1]) == 2, ''
     assert keeper.token.get_token_balance(
         keeper.escrow_payment_condition.address
     ) == starting_balance, ''
@@ -159,9 +161,9 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     print('Downloading NFT content, DID: ' + ddo.did)
 
     success = keeper.nft_access_template.create_agreement(
-        nft_access_agreement_id,
+        nft_access_agreement_id[0],
         asset_id,
-        [nft_holder_cond_id, nft_access_cond_id],
+        [nft_holder_cond_id[0], nft_access_cond_id[0]],
         nft_access_service_agreement.conditions_timelocks,
         nft_access_service_agreement.conditions_timeouts,
         consumer_acc.address,
@@ -170,7 +172,7 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     print('Create NFT Access agreement: ', success)
     assert success, f'createAgreement failed {success}'
     event = keeper.nft_access_template.subscribe_agreement_created(
-        nft_access_agreement_id,
+        nft_access_agreement_id[1],
         10,
         log_event(keeper.nft_access_template.AGREEMENT_CREATED_EVENT),
         (),
@@ -180,12 +182,12 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
 
     # Fulfill nft_holder_condition
     tx_hash = keeper.nft_holder_condition.fulfill(
-        nft_access_agreement_id, asset_id, consumer_acc.address, number_nfts, publisher_acc
+        nft_access_agreement_id[1], asset_id, consumer_acc.address, number_nfts, publisher_acc
     )
 
     keeper.nft_holder_condition.get_tx_receipt(tx_hash)
     event = keeper.nft_holder_condition.subscribe_condition_fulfilled(
-        nft_access_agreement_id,
+        nft_access_agreement_id[1],
         10,
         log_event(keeper.nft_holder_condition.FULFILLED_EVENT),
         (),
@@ -193,15 +195,15 @@ def test_nft_sales_flow(setup_nft_sales_agreements_environment):
     )
 
     tx_hash = keeper.nft_access_condition.fulfill(
-        nft_access_agreement_id, asset_id, consumer_acc.address, publisher_acc
+        nft_access_agreement_id[1], asset_id, consumer_acc.address, publisher_acc
     )
     keeper.nft_access_condition.get_tx_receipt(tx_hash)
     event = keeper.nft_access_condition.subscribe_condition_fulfilled(
-        nft_access_agreement_id,
+        nft_access_agreement_id[1],
         20,
         log_event(keeper.nft_access_condition.FULFILLED_EVENT),
         (),
         wait=True
     )
     assert event, 'no event for AccessCondition.Fulfilled'
-    assert keeper.condition_manager.get_condition_state(access_cond_id) == 2, ''
+    assert keeper.condition_manager.get_condition_state(access_cond_id[1]) == 2, ''
